/*******************************************************************************
* Copyright Reticular Systems Inc. 1997    All rights reserved.
* Proprietary Information, Property of Reticular Systems Inc.
* Filename : HelloWorldFrame.java
*
*
* RCS: $Revision: 1.5 $  $Date: 1998/05/06 17:06:47 $  $Author: donohoe $ 
* Created by: Jim Donohoe -- Tue Sep  9 16:44:59 1997
*******************************************************************************/

package com.reticular.agents.helloWorld;

import java.awt.*;
import java.awt.event.*;
import java.util.Date;

import com.reticular.agentBuilder.agent.perception.PacCommSystem;
import com.reticular.agentBuilder.agent.perception.KqmlMessage;

/*******************************************************************************
* Description : HelloWorldFrame provides a simple interface for the HelloWorld
* agent. This interface has a scrollable text area, a print button, and a quit
* button.
*
* When the user pushes the print button:
*   (1) the callback method actionPerformed will send a KQML message to the
*       agent,
*   (2) the agent will create a string composed of its "Hello World" greeting
*       and its belief about the current time,
*   (3) the agent will invoke the HelloWorldFrame::print method to print the
*       string in the interface text area.
*
* When the user pushes the quit button:
*   (1) the callback method actionPerformed will send a KQML message to the
*       agent,
*   (2) the message will trigger execution of the built-in action
*       ShutdownEngine, which will close all files and shutdown the engine.
*
* Of course, for this application all the work could be done in the callback
* code, without an agent.  That would be simpler, but not nearly as much fun.
* 
* @author Jim Donohoe
*******************************************************************************/
public class HelloWorldFrame extends Frame
   implements Runnable, Cloneable, ActionListener
{ 

//PUBLIC DATA
   public static final int FRAME_WIDTH = 800;
   public static final int FRAME_HEIGHT = 600;
   public static final String FRAME_TITLE = "Hello World";
      
   public static final int TEXT_AREA_ROWS = 8;
   public static final int TEXT_AREA_COLS = 48;
   public static final String EMPTY_STRING = "";
   public static final String PRINT_BUTTON_LABEL = "    Say Hello    ";
   public static final String QUIT_BUTTON_LABEL =  "       Quit      ";

   public static final String BUTTON_FONT_NAME = "SansSerif";
   public static final int BUTTON_FONT_STYLE = Font.BOLD;
   public static final int BUTTON_FONT_SIZE = 18;

   public static final String TEXT_FONT_NAME = "SansSerif";
   public static final int TEXT_FONT_STYLE = Font.PLAIN;
   public static final int TEXT_FONT_SIZE = 16;


//PRIVATE DATA
   /** The text area will display all the output supplied by the agent. **/
   private TextArea _textArea;

   /** Reference to the PacCommSystem object in the HelloWorldFrame object.
       This encapsulates all the low-level information needed to send KQML
       messages from this control panel back to the agent. **/
   private PacCommSystem _communicationSystem;

   
/********************************PUBLIC METHODS********************************
                                 

/**
  * Description : main() is used for development and testing the interface.
  * This allows the interface to be displayed without a running agent, by
  * invoking: java HelloWorldFrame
  *
  * NOTE: This method is not used in the normal operation of the interface
  * when the interface is constructed by an agent.  This creates a dummy
  * interface with non-functional buttons.
  *
  * @param argv - String[]
  * @return void
  * @author Jim Donohoe
  */
   public static void main( String[] argv )
   {
      // Create a frame with a null PacCommSystemConfig object.  This will
      // create an interface without any functionality; it's only for testing
      // the look of the interface.
      HelloWorldFrame testFrame = new HelloWorldFrame( null );

      // Invoke the print method on this object. This print(String) is the
      // same method that gets called by the agent engine to implement the
      // agent's private action Print(String).
      testFrame.print( "This is a test interface for the HelloWorld " +
                       "application;" );
      testFrame.print( "the buttons are not functional.  Example output:\n" );

      testFrame.print( "Hello World! The time is " +
                       new Date( System.currentTimeMillis() ).toString() );

      // Display the interface.  This will display the interface on the same
      // thread as main(), but that's not a problem here because we're only
      // testing the interface so there is no agent code getting blocked.
      testFrame.run();
   }


                                 
/**
  * Description : HelloWorldFrame constructor
  *  
  * @param  pacCommSystem - a PacCommSystem object supplied by the parent
  * agent, to enable this PAC to send KQML messages to the agent.
  * @author Jim Donohoe
  */
   public  HelloWorldFrame( PacCommSystem pacCommSystem )
   {
      _communicationSystem = pacCommSystem;
   }

   
/**
  * Description : Append the string argument to the text area.  This will
  * cause the string to be displayed in the interface.
  * 
  * @param stringFromAgent - text supplied by the agent via a private action
  * argument.
  * @return void  
  * @author Jim Donohoe
  */
   public void print( String stringFromAgent )
   {
      _textArea.append( stringFromAgent + "\n" );
   }

    
   
   
/**
  * Description : Build and display the interface.  run() is the method which
  * must be defined in order to implement the Runnable interface.  This method
  * will be invoked when the agent's private action Start() is executed on
  * a separate thread.  The agent engine will create a java.lang.Thread
  * object with the helloWorldFrame class (which implements Runnable) as an
  * argument, then invoke Thread::start(), which will then automatically
  * invoke this run() method.  The following code is equivalent to the code
  * used in the agent engine to execute a separately-threaded private action.
  * 
  *   Thread separateThread = new Thread( (Runnable) helloWorldFrame );
  *   separateThread.start();
  *
  * The result will be that the code defined here in run() will execute on a
  * separate thread from the rest of the agent engine.  This way the interface
  * can be built and displayed without blocking execution of the engine.
  * 
  * @return void  
  * @author Jim Donohoe
  */
   public void run( )
   {
      //-----------------------------------------------------------------------
      //            Set up the frame layout and common fonts 
      //-----------------------------------------------------------------------
   
      GridBagLayout frameLayout = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      this.setLayout( frameLayout );

      FlowLayout buttonsLayout = new FlowLayout();
      Panel buttonsPanel = new Panel( buttonsLayout );

      Font textFont = new Font( TEXT_FONT_NAME, TEXT_FONT_STYLE,
                                TEXT_FONT_SIZE );

      Font buttonFont = new Font( BUTTON_FONT_NAME, BUTTON_FONT_STYLE,
                                  BUTTON_FONT_SIZE );

      //-----------------------------------------------------------------------
      //          Create the text area for the agent's responses
      //-----------------------------------------------------------------------

      _textArea = new TextArea( EMPTY_STRING,
                                TEXT_AREA_ROWS,
                                TEXT_AREA_COLS,
                                TextArea.SCROLLBARS_VERTICAL_ONLY );
      _textArea.setEditable( false );
      _textArea.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 0;
      frameLayout.setConstraints( _textArea, constraints );
      this.add( _textArea );
      
      //-----------------------------------------------------------------------
      //        Create buttons to tell the agent to print or quit 
      //-----------------------------------------------------------------------

      Button printButton = new Button( PRINT_BUTTON_LABEL );
      printButton.setFont( buttonFont );
      printButton.setActionCommand( PRINT_BUTTON_LABEL );
      printButton.addActionListener( this );
      buttonsPanel.add( printButton );

      Button quitButton = new Button( QUIT_BUTTON_LABEL );
      quitButton.setFont( buttonFont );
      quitButton.setActionCommand( QUIT_BUTTON_LABEL );
      quitButton.addActionListener( this );
      buttonsPanel.add( quitButton );
   
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 1;
      frameLayout.setConstraints( buttonsPanel, constraints );
      this.add( buttonsPanel );

      this.setTitle( FRAME_TITLE );

      //-----------------------------------------------------------------------
      //        Handle the 'Close' signal from the window manager
      //-----------------------------------------------------------------------

      // Add a window adapter to listen for a signal from the window manager,
      // in case the user chooses Close from the menu.
      WindowAdapter closeWindowAdapter = new WindowAdapter()
         {
            public void windowClosing( WindowEvent windowEvent )
            {
               System.out.println("Quitting HelloWorld Application.");
               dispose();
            }
         };
      this.addWindowListener( closeWindowAdapter );
      
      //-----------------------------------------------------------------------
      //                    Display the interface
      //-----------------------------------------------------------------------

      // Invoke superclass methods Window::pack and Window::show.
      this.pack();
      this.show();   
   }


/**
  * Description : Build a KQML message and send it to the parent agent. 
  * This class (HelloWorldFrame) is registered as an action listener for the
  * two buttons on its interface, so this method will be invoked by the Java
  * event-handling subsystem when either button gets pushed.
  *  
  * @param event - ActionEvent object passed into this method by the Java
  * event-handling subsystem.
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void actionPerformed( ActionEvent event )
   {
      //-----------------------------------------------------------------------
      // Instantiate an empty KQML message, then fill it in with the values
      // that are expected by the agent for each type of event.  For either
      // the print or quit command the KQML performative is "achieve".  The
      // message content will differ depending on the command.
      //
      // Note that the content of the message must match the value expected by
      // the agent in order for the message to have the desired effect.  When
      // this application was designed the message protocol for the agent and
      // this control panel was defined.  A message with Performative="achieve"
      // and a string Content="Say Hello" will be accepted by a rule in the
      // agent and interpreted to be a print command.  Similarly, a message
      // with Performative="achieve" and a string Content="Quit" will be
      // accepted by a second rule and interpreted to be a quit command.
      //-----------------------------------------------------------------------

      KqmlMessage message = new KqmlMessage();

      //-----------------------------------------------------------------------
      // Use the action command stored in the ActionEvent object to distinguish
      // between the two types of button pushes that could have caused this
      // invocation.  The values for these action command constants are defined
      // at the top of this file, and the values are set in the run() method
      // when this HelloWorldFrame gets registered as an event listener for
      // the buttons.
      //-----------------------------------------------------------------------
      
      if( event.getActionCommand().equals( PRINT_BUTTON_LABEL ))
      {
         message.setPerformative("achieve");
         message.setContent("Say Hello");
      }
      else if( event.getActionCommand().equals( QUIT_BUTTON_LABEL ))
      {
         message.setPerformative("achieve");
         message.setContent("Quit");
      }
      
      //-----------------------------------------------------------------------
      // Send the message from this PAC object to the parent agent.  The
      // addressing and sending of the message is handled by the communication
      // system. 
      //-----------------------------------------------------------------------

      if( _communicationSystem != null )
      {
         _communicationSystem.sendKqmlMessageToAgent( message );
      }
   }
   

/**
  * Description : Clone the HelloWorldFrame object.  This is included here
  * to satisfy the Cloneable interface, even though we don't really clone
  * the object.  Any PAC objects need to implement Cloneable, because objects
  * get cloned before they are asserted into the mental state or used as args
  * in private actions.  When HelloWorldFrame::clone gets called we just
  * return a reference to the object itself, since we don't want to make
  * duplicate copies of the interface object.
  * 
  * @return Object   
  * @author Jim Donohoe
  */
   public Object clone()
   {      
      return this;
   }

   
}

/*******************************************************************************
 End of : HelloWorldFrame.java
*******************************************************************************/
